unit uRtfTasks;

{$INCLUDE 'RTFConv.inc'}
{$DEFINE QUICK_AND_DIRTY}

interface

uses
  SysUtils, Classes,
  uRtf, uAdvRtf, uRtfTokens;

type
  TRtfTasks = class(TAdvRtf)
    private
    protected
      {$IFDEF QUICK_AND_DIRTY}
      procedure CountCharactersForSizes(Sizes: TStringList; CurrentSize: integer; var TokenIndex: integer); virtual;
      {$ENDIF}
    public
      constructor Create; override;
      destructor Destroy; override;
      // Change font sizes so that normal text has size TargetSize (in half-points!!!)
      function Resize(const TargetSize: integer): boolean; virtual;
      // Justify left-aligned text
      function Justify: boolean; virtual;
    end;

implementation

{ TRtfTasks }

constructor TRtfTasks.Create;
begin
  inherited;
end;

destructor TRtfTasks.Destroy;
begin
  inherited;
end;

{$IFDEF QUICK_AND_DIRTY}
procedure TRtfTasks.CountCharactersForSizes(Sizes: TStringList; CurrentSize: integer; var TokenIndex: integer);
var n: integer;
begin
  n := Pred(Tokens.Count);
  while TokenIndex <= n do
    if IsComment(TokenIndex) then
      begin
      SkipGroup(TokenIndex);
      Exit;
      end
    else
      with Tokens[TokenIndex] do
        if TokenType = rtfGroupStart then
          begin
          Inc(TokenIndex);
          CountCharactersForSizes(Sizes, CurrentSize, TokenIndex);
          end
        else if TokenType = rtfGroupEnd then
          begin
          Inc(TokenIndex);
          Exit;
          end
        else if (TokenType = rtfControlWord) and
                ((Token = '\fonttbl') or (Token = '\stylesheet') or (Token = '\colortbl') or (Token = '\listtable') or (Token = '\info'))
        then
          begin
          SkipGroup(TokenIndex);
          Exit;
          end
        else if (TokenType = rtfControlWordWithParam) and (Token = '\fs') then
          begin
          CurrentSize := Sizes.IndexOf(Parameter);
          if CurrentSize < 0 then
            CurrentSize := Sizes.AddObject(Parameter, Pointer(0));
          Inc(TokenIndex);
          end
        else if TokenType = rtfPlainText then
          begin
          Sizes.Objects[CurrentSize] := Pointer(Integer(Sizes.Objects[CurrentSize]) + Length(Token));
          Inc(TokenIndex);
          end
        else
          Inc(TokenIndex);
end;
{$ENDIF}

function TRtfTasks.Resize(const TargetSize: integer): boolean;
{$IFDEF QUICK_AND_DIRTY}
  var Sizes: TStringList;
      CurrentSize, MaxSize, MaxSizeCount: integer;
      i: integer;
{$ENDIF}
begin
  {$IFDEF QUICK_AND_DIRTY}
    Sizes := TStringList.Create;
    try
      Sizes.Add('20');
      CurrentSize := 0;
      i := 0;
      CountCharactersForSizes(Sizes, CurrentSize, i);
      // Tady mam spocitane, kolik znaku textu odpovida kazde velikosti pisma.
      // Ted si zjistim nejcetnejsi velikost a tu budu povazovat za zakladni pismo
      MaxSize := 0;
      MaxSizeCount := Integer(Sizes.Objects[0]);
      for i := 1 to Pred(Sizes.Count) do
        if Integer(Sizes.Objects[i]) > MaxSizeCount then
          begin
          MaxSize := i;
          MaxSizeCount := Integer(Sizes.Objects[i]);
          end;
      MaxSize := StrToInt(Sizes[MaxSize]);
      // Ted mam zadanou velikost a normalni velikost, takze muzu prepocitavat
      if MaxSize <> TargetSize then
        for i := Pred(Tokens.Count) downto 0 do
          with Tokens[i] do
            if (TokenType = rtfControlWordWithParam) and (Token = '\fs') then
              begin
              Parameter := IntToStr(StrToInt(Parameter) * TargetSize div MaxSize);
              end
            else if (TokenType = rtfControlWord) and ((Token = '\par') or (Token = '\pard')) then
              begin
              Tokens.Append(i, TRtfToken.Create(rtfControlWordWithParam, '\fs', IntToStr(TargetSize)));
              end;
      Result := true;
    finally
      Sizes.Free;
      end;
  {$ENDIF}
  if Tokens.Inserted then
    Build;
end;

function TRtfTasks.Justify: boolean;
{$IFDEF QUICK_AND_DIRTY}
var i: integer;
{$ENDIF}
begin
  {$IFDEF QUICK_AND_DIRTY}
    for i := Pred(Tokens.Count) downto 0 do
      with Tokens[i] do
        if (TokenType = rtfControlWord) and (Token = '\ql') then
          Token := '\qj'
        else if (TokenType = rtfControlWordWithParam) and ((Token = '\s') or (Token = '\ds')) then
          Tokens.Append(i, TRtfToken.Create(rtfControlWord, '\qj', ''))
        else if (TokenType = rtfControlWord) and ((Token = '\par') or (Token = '\pard')) then
          Tokens.Append(i, TRtfToken.Create(rtfControlWord, '\qj', ''));
    Result := True;
  {$ENDIF}
  if Tokens.Inserted then
    Build;
end;

end.
